/* ScicosLab interfacing function of the Scicos standalone */
/*     Generated by Code_Generation toolbox of Scicos with scicos4.4 */
/*     date : 20-Oct-2010 */

/* Copyright (c) 1989-2010 Metalau project INRIA */

/* ---- Headers ---- */
#include <string.h>
#include <stdlib.h>
#include "stack-c.h"
#include "mex.h"

/* external defintion of standalone simulation function */
extern int SuperBlock_sim(double tf, double dt, double h, int solver, \
                          int *typin, void **inptr, int *typout, void **outptr);

/* external definition of error table function */
extern void get_err_msg(int ierr,char *err_msg);

#if WIN32
  #ifndef coserr
    #define coserr _imp__coserr
  #endif
#endif

/* standalone still use scicos.c here ! */
extern struct {char buf[4096];} coserr;

/* structure definition of in/out sensors/actuators */
typedef struct {
  int typ;      /* data type */
  int ndims;    /* number of dims */
  int ndata;    /* number of data */
  int *dims;    /* size of data (length ndims) */
  double *time; /* date of data (length ndata) */
  void *data;   /* data (length ndata*prod(dims)) */
} scicos_inout;

/* [out1]=SuperBlock(in1,[,te][,tf][,h][,solver])
 *
 * Rhs :
 * in     : input signal(s) for sensors
 * te     : sampling time (default : 0.1)
 * tf     : final time (default : 30)
 * h      : solver step (default : 0.001)
 * solver : type of solver (1:Euler, 2:Heun, 3:R.Kutta 4th order)
 *          (default : 3)
 *
 * Lhs :
 * out    : output signal(s) coming from actuator
 */
int intSuperBlock_sci(fname,fname_len)
   char *fname;
   unsigned long fname_len;
{
  /* variables to handle ptr and dims coming from scilab stack */
  static int l1,n1,m1;  /* sensor 1 */
  static int l2,n2,m2;  /* te */
  static int l3,n3,m3;  /* tf */
  static int l4,n4,m4;  /* h */
  static int l5,n5,m5;  /* solver */
  static int l6,n6,m6;  /* actuator 1 */
  static int mL=4,nL=1,lL; /* def scicos signal */
  static char *Str[]={"st","dims","values","time"};
  static int V[]={1,1};
  static SciIntMat M={1,2,4,-1,V};  /* dims field */

  /* variables for standalone simulation function */
  /* default values te,tf,h and solver */
  double te  = 0.1;
  double tf  = 30;
  double h   = 0.001;
  int solver = 3;

  /* Inputs of sensors */
  /* int nin=1; */
  int typin[]={2};
  void *inptr[]={0};
  scicos_inout in_1;

  /* Outputs of actuators */
  /* int nout=1; */
  int typout[]={2};
  void *outptr[]={0};
  scicos_inout out_1;

  /* ptr for output transposition */
  void *out_ptr1;

  /* Ouput standalone error handling */
  int ierr;
  char err_msg[4096];

  /* counter local variable */
  int i,j;

  /* check numbers of rhs/lhs */
  CheckRhs(1,5);
  CheckLhs(1,1);

  switch(Rhs) {
    case 5 :    /* check/get solver */
       GetRhsVar(5,"d",&m5,&n5,&l5);
       CheckDims(5,m5,n5,1,1);
       solver = (int) *stk(l5);

    case 4 :    /* check/get h */
       GetRhsVar(4,"d",&m4,&n4,&l4);
       CheckDims(4,m4,n4,1,1);
       h = *stk(l4);

    case 3 :    /* check/get tf */
       GetRhsVar(3,"d",&m3,&n3,&l3);
       CheckDims(3,m3,n3,1,1);
       tf = *stk(l3);

    case 2 :    /* check/get te */
       GetRhsVar(2,"d",&m2,&n2,&l2);
       CheckDims(2,m2,n2,1,1);
       te = *stk(l2);

    case 1 :    /* check/get sensor 1 */
       GetRhsVar(1,"d",&m1,&n1,&l1);
       /* Many thing to do */

  }

  /* inform in struct of sensor 1 */
  in_1.typ       = 10;
  in_1.ndims     = 2;
  in_1.ndata     = m1;
  if ((in_1.dims = (int*) malloc(in_1.ndims*sizeof(int)))==NULL){
    return 0;
  }
  in_1.dims[0] = 1;
  in_1.dims[1] = n1;
  if ((in_1.data = (double *) malloc(in_1.ndata*in_1.dims[0]*in_1.dims[1]*sizeof(double)))==NULL){
    free(in_1.dims);
    return 0;
  }
  if ((in_1.dims[0] == 1) &&  (in_1.dims[1] != 1)) {
    for(i=0;i<in_1.ndata;i++) {
      for(j=0;j<in_1.dims[1];j++) {
         *((double *)in_1.data + i*in_1.dims[1] + j) = \
             *((double *) stk(l1) + j*in_1.ndata + i);
      }
    }
  }
  else {
    for(i=0;i<in_1.ndata*in_1.dims[0]*in_1.dims[1];i++) {
      *((double *)in_1.data + i) = *((double *) stk(l1) + i);
    }
  }

  /* inform out struct of actuator 1 */
  out_1.typ       = 10;
  out_1.ndims     = 2;
  out_1.ndata     = 1000; /* default buffer size */
  if ((out_1.dims = (int *) malloc(out_1.ndims*sizeof(int)))==NULL){
    free(in_1.data);
    free(in_1.dims);
    return 0;
  }
  out_1.dims[0]   = 1;
  out_1.dims[1]   = 1;
  out_1.data      = NULL;
  if ((out_1.data = (double *) realloc(out_1.data, \
       out_1.ndata*out_1.dims[0]*out_1.dims[1]*sizeof(double)))==NULL){
    free(in_1.data);
    free(in_1.dims);
    free(out_1.dims);
    return 0;
  }
  out_1.time      = NULL;
  if ((out_1.time = (double *) realloc(out_1.time,out_1.ndata*sizeof(double)))==NULL){
    free(in_1.data);
    free(in_1.dims);
    free(out_1.dims);
    free(out_1.data);
    return 0;
  }

  /* store ptr of sensors/actuators in inptr/outptr */
  inptr[0]  = &in_1;
  outptr[0] = &out_1;

  /* call standalone simulation function */
  ierr=SuperBlock_sim(tf, te, h, solver, typin, inptr, typout, outptr);

  /* display error message */
  if (ierr!=0) {
    /* standalone still use scicos.c here ! */
    if(ierr!=10) {
      get_err_msg(ierr,err_msg);
    }
    else {
      strcpy(err_msg,coserr.buf);
    }
    sciprint("Simulation fails with error number %d :\n%s\n",ierr,err_msg);
  }

  /* Create LhsVar */
  /* actuator 1 */
  CreateVar(6, "m", &mL, &nL, &lL);              /* mlist */
  CreateListVarFromPtr(6,1,"S", &nL, &mL, Str);  /* str of scicos signal */
  CreateListVarFromPtr(6,2,"I", &M.m, &M.n, &M); /* def dims of mlist */
  m6=out_1.ndata;
  /* vector case */
  if ((out_1.dims[0]==1 && out_1.dims[1]!=1) || \
      (out_1.dims[0]!=1 && out_1.dims[1]==1)) {
    if (out_1.dims[0]!=1) {
      n6=out_1.dims[0];
    }
    else {
      n6=out_1.dims[1];
    }
    /*transpose output */
    if ((out_ptr1 = (double *) malloc(m6*n6*sizeof(double)))==NULL){
      free(in_1.data);
      free(in_1.dims);
      free(out_1.dims);
      free(out_1.data);
      free(out_1.time);
      return 0;
    }
    for(i=0;i<m6;i++) {
      for(j=0;j<n6;j++) {
        *((double *)out_ptr1 + j*m6 + i) = *((double *) out_1.data + i*n6 + j);
      }
    }
    CreateListVarFromPtr(6,3,"d", &m6, &n6, &out_ptr1);   /* data */
  }
  else {
    /* scalar case */
    if (out_1.dims[0]==1 && out_1.dims[1]==1) {
      n6=1;
      CreateListVarFromPtr(6,3,"d", &m6, &n6, &out_1.data);   /* data */
    }
    /* matrix case : TODO
     * note that for that time,
     * matrix are concatened in vectors.
     */
    else {
      n6=out_1.dims[0]*out_1.dims[1];
      /*transpose output */
      if ((out_ptr1 = (double *) malloc(m6*n6*sizeof(double)))==NULL){
        free(in_1.data);
        free(in_1.dims);
        free(out_1.dims);
        free(out_1.data);
        free(out_1.time);
        return 0;
      }
      for(i=0;i<m6;i++) {
        for(j=0;j<n6;j++) {
          *((double *)out_ptr1 + j*m6 + i) = *((double *) out_1.data + i*n6 + j);
        }
      }
      CreateListVarFromPtr(6,3,"d", &m6, &n6, &out_ptr1);   /* data */
    }
  }
  n6=1;
  CreateListVarFromPtr(6,4,"d", &m6, &n6, &out_1.time);   /* event date */
  /* Many thing to do */

  /* Put LhsVar */
  LhsVar(1)=6;

  /* free allocated array */
  free(in_1.data);
  free(in_1.dims);

  free(out_1.dims);
  free(out_1.data);
  free(out_1.time);
  if (n6 != 1) {
    free(out_ptr1);
  }
  return 0;
}

static GenericTable Tab[]=
{
 {(Myinterfun)sci_gateway,intSuperBlock_sci,
 "SuperBlock"}
};

int C2F(intSuperBlock_sci)()
{
 Rhs = Max(0, Rhs);
 (*(Tab[Fin-1].f))(Tab[Fin-1].name,Tab[Fin-1].F);
 return 0;
}

